;+
; :Description:
;    Describe the procedure.
;    calculate covariance=E(x*y)-E(x)*E(y);x：first soil moisture product，y：second soil moisture product
;    E(x*y),E(x),E(y)分别为在时间段2012-2017年间，x*y平均影像，x平均影像，y平均影像
; Params:
;    outfiledir
;    D_A
;    file_SM1
;    file_SM2
; :Author: xieqiuxia
;-
pro TCA_RMSE

  ;设置输出影像路径
  outfiledir='J:\Merging_SM_Products_Fusion_ESACCI_SMAP_2\TCA_P\'
  D_A='D'
  file_SM1='Fusion'
  file_SM2='ESACCI'
  file_SM3='SMAP'
  
  out_name=outfiledir+file_SM1+'_2015_2017'+'_TCA_RMSE_'+D_A+'.tif
  out_name1=outfiledir+file_SM2+'_2015_2017'+'_TCA_RMSE_'+D_A+'.tif
  out_name2=outfiledir+file_SM3+'_2015_2017'+'_TCA_RMSE_'+D_A+'.tif
  
  return=RMSE(outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name);计算file_SM1的RMSE
  return1=RMSE1(outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name1);计算file_SM2的RMSE
  return2=RMSE2(outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name2);计算file_SM3的RMSE
  print,return
  print,return1
  print,return2

end

;计算写方差函数
function RMSE,outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name
  COMPILE_OPT idl2
  ENVI,/restore_base_save_files
  envi_batch_init,LOG_FILE='batch.log'

  ;image_dir=DIALOG_PICKFILE(path=outfiledir,/Directory)

  ;读取第一景影像
  name1=file_SM1+'_'+file_SM1+'*'+'Covariance'+'*'+D_A+'.tif
  image_files1=file_search(outfiledir,name1,count=n1)
  ENVI_OPEN_FILE,image_files1,r_fid=fid1, /no_interactive_query, /no_realize
  if fid1 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid1,file_type=file_type1,nl=nl1,ns=ns1,dims=dims1,nb=nb1
  out_proj1 = envi_get_projection(fid=fid1,pixel_size=out_ps1)
  map_info1= envi_get_map_info(fid=fid1)
  print,image_files1

  ;读取第二景影像
  name2=file_SM1+'_'+file_SM2+'*'+'Covariance'+'*'+D_A+'.tif
  image_files2=file_search(outfiledir,name2,count=n2)
  ENVI_OPEN_FILE,image_files2,r_fid=fid2, /no_interactive_query, /no_realize
  if fid2 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid2,file_type=file_type2,nl=nl2,ns=ns2,dims=dims2,nb=nb2
  out_proj2 = envi_get_projection(fid=fid2,pixel_size=out_ps2)
  map_info2= envi_get_map_info(fid=fid2)
  print,image_files2

  ;读取第三景影像
  name3=file_SM1+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files3=file_search(outfiledir,name3,count=n3)
  ENVI_OPEN_FILE,image_files3,r_fid=fid3, /no_interactive_query, /no_realize
  if fid3 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid3,file_type=file_type3,nl=nl3,ns=ns3,dims=dims3,nb=nb3
  out_proj3 = envi_get_projection(fid=fid3,pixel_size=out_ps3)
  map_info3= envi_get_map_info(fid=fid3)
  print,image_files3
  
  ;读取第四景影像
  name4=file_SM2+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files4=file_search(outfiledir,name4,count=n4)
  ENVI_OPEN_FILE,image_files4,r_fid=fid4, /no_interactive_query, /no_realize
  if fid4 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid4,file_type=file_type4,nl=nl4,ns=ns4,dims=dims4,nb=nb4
  out_proj4 = envi_get_projection(fid=fid4,pixel_size=out_ps4)
  map_info4= envi_get_map_info(fid=fid4)
  print,image_files4

  ;对第一景和第二景影像进行layerstachking
  ;设置数组
  fid = indgen(4)
  pos = indgen(4)
  dims = indgen(5,4)

  pos[0] = 0
  pos[1] = 0
  pos[2] = 0
  pos[3] = 0
  
  fid[0] = fid1
  fid[1] = fid2
  fid[2] = fid3
  fid[3] = fid3
  
  dims[0,0] = [-1,0,ns1-1,0,nl1-1]
  dims[0,1] = [-1,0,ns2-1,0,nl2-1]
  dims[0,2] = [-1,0,ns3-1,0,nl3-1]
  dims[0,3] = [-1,0,ns4-1,0,nl4-1]

  ;计算协方差
  ;bandmath and layerstacking
  envi_doit, 'envi_layer_stacking_doit', $
    fid=[fid1,fid2,fid3,fid4],pos=pos, dims=dims, $
    out_dt=5,/in_memory, $
    interp=0,out_ps=out_ps1, $
    out_proj=out_proj1, r_fid=r_fid


  exp='sqrt(abs(float(b1)-float(b2)*float(b3)/float(b4)))'
  envi_doit,'math_doit', $
    fid=[r_fid,r_fid,r_fid,r_fid],pos=[0,1,2,3],dims=dims, $
    exp=exp,out_name=out_name, $
    r_fid=fid_math
  
  print,out_name
  return,'...................ok.....................'
end

;计算写方差函数
function RMSE1,outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name
  COMPILE_OPT idl2
  ENVI,/restore_base_save_files
  envi_batch_init,LOG_FILE='batch.log'

  ;image_dir=DIALOG_PICKFILE(path=outfiledir,/Directory)

  ;读取第一景影像
  name1=file_SM2+'_'+file_SM2+'*'+'Covariance'+'*'+D_A+'.tif
  image_files1=file_search(outfiledir,name1,count=n1)
  ENVI_OPEN_FILE,image_files1,r_fid=fid1, /no_interactive_query, /no_realize
  if fid1 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid1,file_type=file_type1,nl=nl1,ns=ns1,dims=dims1,nb=nb1
  out_proj1 = envi_get_projection(fid=fid1,pixel_size=out_ps1)
  map_info1= envi_get_map_info(fid=fid1)
  print,image_files1

  ;读取第二景影像
  name2=file_SM1+'_'+file_SM2+'*'+'Covariance'+'*'+D_A+'.tif
  image_files2=file_search(outfiledir,name2,count=n2)
  ENVI_OPEN_FILE,image_files2,r_fid=fid2, /no_interactive_query, /no_realize
  if fid2 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid2,file_type=file_type2,nl=nl2,ns=ns2,dims=dims2,nb=nb2
  out_proj2 = envi_get_projection(fid=fid2,pixel_size=out_ps2)
  map_info2= envi_get_map_info(fid=fid2)
  print,image_files2

  ;读取第三景影像
  name3=file_SM2+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files3=file_search(outfiledir,name3,count=n3)
  ENVI_OPEN_FILE,image_files3,r_fid=fid3, /no_interactive_query, /no_realize
  if fid3 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid3,file_type=file_type3,nl=nl3,ns=ns3,dims=dims3,nb=nb3
  out_proj3 = envi_get_projection(fid=fid3,pixel_size=out_ps3)
  map_info3= envi_get_map_info(fid=fid3)
  print,image_files3

  ;读取第四景影像
  name4=file_SM1+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files4=file_search(outfiledir,name4,count=n4)
  ENVI_OPEN_FILE,image_files4,r_fid=fid4, /no_interactive_query, /no_realize
  if fid4 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid4,file_type=file_type4,nl=nl4,ns=ns4,dims=dims4,nb=nb4
  out_proj4 = envi_get_projection(fid=fid4,pixel_size=out_ps4)
  map_info4= envi_get_map_info(fid=fid4)
  print,image_files4

  ;对第一景和第二景影像进行layerstachking
  ;设置数组
  fid = indgen(4)
  pos = indgen(4)
  dims = indgen(5,4)

  pos[0] = 0
  pos[1] = 0
  pos[2] = 0
  pos[3] = 0

  fid[0] = fid1
  fid[1] = fid2
  fid[2] = fid3
  fid[3] = fid3

  dims[0,0] = [-1,0,ns1-1,0,nl1-1]
  dims[0,1] = [-1,0,ns2-1,0,nl2-1]
  dims[0,2] = [-1,0,ns3-1,0,nl3-1]
  dims[0,3] = [-1,0,ns4-1,0,nl4-1]

  ;计算协方差
  ;bandmath and layerstacking
  envi_doit, 'envi_layer_stacking_doit', $
    fid=[fid1,fid2,fid3,fid4],pos=pos, dims=dims, $
    out_dt=5,/in_memory, $
    interp=0,out_ps=out_ps1, $
    out_proj=out_proj1, r_fid=r_fid


  exp='sqrt(abs(float(b1)-float(b2)*float(b3)/float(b4)))'
  envi_doit,'math_doit', $
    fid=[r_fid,r_fid,r_fid,r_fid],pos=[0,1,2,3],dims=dims, $
    exp=exp,out_name=out_name, $
    r_fid=fid_math

  print,out_name
  return,'...................ok.....................'
end

;计算写方差函数
function RMSE2,outfiledir,D_A,file_SM1,file_SM2,file_SM3,out_name
  COMPILE_OPT idl2
  ENVI,/restore_base_save_files
  envi_batch_init,LOG_FILE='batch.log'

  ;image_dir=DIALOG_PICKFILE(path=outfiledir,/Directory)

  ;读取第一景影像
  name1=file_SM3+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files1=file_search(outfiledir,name1,count=n1)
  ENVI_OPEN_FILE,image_files1,r_fid=fid1, /no_interactive_query, /no_realize
  if fid1 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid1,file_type=file_type1,nl=nl1,ns=ns1,dims=dims1,nb=nb1
  out_proj1 = envi_get_projection(fid=fid1,pixel_size=out_ps1)
  map_info1= envi_get_map_info(fid=fid1)
  print,image_files1

  ;读取第二景影像
  name2=file_SM1+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files2=file_search(outfiledir,name2,count=n2)
  ENVI_OPEN_FILE,image_files2,r_fid=fid2, /no_interactive_query, /no_realize
  if fid2 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid2,file_type=file_type2,nl=nl2,ns=ns2,dims=dims2,nb=nb2
  out_proj2 = envi_get_projection(fid=fid2,pixel_size=out_ps2)
  map_info2= envi_get_map_info(fid=fid2)
  print,image_files2

  ;读取第三景影像
  name3=file_SM2+'_'+file_SM3+'*'+'Covariance'+'*'+D_A+'.tif
  image_files3=file_search(outfiledir,name3,count=n3)
  ENVI_OPEN_FILE,image_files3,r_fid=fid3, /no_interactive_query, /no_realize
  if fid3 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid3,file_type=file_type3,nl=nl3,ns=ns3,dims=dims3,nb=nb3
  out_proj3 = envi_get_projection(fid=fid3,pixel_size=out_ps3)
  map_info3= envi_get_map_info(fid=fid3)
  print,image_files3

  ;读取第四景影像
  name4=file_SM1+'_'+file_SM2+'*'+'Covariance'+'*'+D_A+'.tif
  image_files4=file_search(outfiledir,name4,count=n4)
  ENVI_OPEN_FILE,image_files4,r_fid=fid4, /no_interactive_query, /no_realize
  if fid4 eq -1 then begin
    print,'It is a mistake'
  endif
  ENVI_FILE_QUERY,fid4,file_type=file_type4,nl=nl4,ns=ns4,dims=dims4,nb=nb4
  out_proj4 = envi_get_projection(fid=fid4,pixel_size=out_ps4)
  map_info4= envi_get_map_info(fid=fid4)
  print,image_files4

  ;对第一景和第二景影像进行layerstachking
  ;设置数组
  fid = indgen(4)
  pos = indgen(4)
  dims = indgen(5,4)

  pos[0] = 0
  pos[1] = 0
  pos[2] = 0
  pos[3] = 0

  fid[0] = fid1
  fid[1] = fid2
  fid[2] = fid3
  fid[3] = fid3

  dims[0,0] = [-1,0,ns1-1,0,nl1-1]
  dims[0,1] = [-1,0,ns2-1,0,nl2-1]
  dims[0,2] = [-1,0,ns3-1,0,nl3-1]
  dims[0,3] = [-1,0,ns4-1,0,nl4-1]

  ;计算协方差
  ;bandmath and layerstacking
  envi_doit, 'envi_layer_stacking_doit', $
    fid=[fid1,fid2,fid3,fid4],pos=pos, dims=dims, $
    out_dt=5,/in_memory, $
    interp=0,out_ps=out_ps1, $
    out_proj=out_proj1, r_fid=r_fid


  exp='sqrt(abs(float(b1)-float(b2)*float(b3)/float(b4)))'
  envi_doit,'math_doit', $
    fid=[r_fid,r_fid,r_fid,r_fid],pos=[0,1,2,3],dims=dims, $
    exp=exp,out_name=out_name, $
    r_fid=fid_math

  print,out_name
  return,'...................ok.....................'
end